@@ -65,7 +65,7 @@ class GLTF(object):
         self.filename = filename
         self.asset = {
             'version': '2.0',
-            'generator': 'Seek GLTF Generator - ' + get_version(),
+            'generator': 'Seek glTF Generator - ' + get_version(),
         }
         self.default_to_glb = binary
 
@@ -300,6 +300,24 @@ class GLTF(object):
             transform = root.matrix.dot(transform)
         root.matrix = transform
 
+    def get_bounding_box(self):
+        g = self.copy()
+        g.apply_all_transforms()
+        g_max = [0, 0, 0]
+        g_min = [0, 0, 0]
+        for m in g.meshes:
+            for p in m.primitives:
+                p_max = p.positions.max
+                g_max[0] = max(g_max[0], p_max[0])
+                g_max[1] = max(g_max[1], p_max[1])
+                g_max[2] = max(g_max[2], p_max[2])
+                p_min = p.positions.min
+                g_min[0] = min(g_min[0], p_min[0])
+                g_min[1] = min(g_min[1], p_min[1])
+                g_min[2] = min(g_min[2], p_min[2])
+
+        return tuple(_max - _min for _max, _min in zip(g_max, g_min))
+
     def copy(self):
         return GLTF.load(self.render())
 
@@ -354,7 +372,8 @@ class GLTF(object):
             data['accessors'] = [acc.render(buffer, self) for acc in self.accessors]
 
         if self.images:
-            data['images'] = [img.render(buffer, self, embed, split_to) for img in self.images]
+            data['images'] = [img.render(buffer, self, embed, split_to, splitting=bool(split_to))
+                              for img in self.images]
 
         for bv in self.buffer_views:
             bv.write_to_buffer()
@@ -581,10 +600,6 @@ class GLTF(object):
                     self.skins.append(skin)
                     break
 
-        # repair animations:
-        for anim in self.animations:
-            anim.repair()
-
         # get rid of duplicate materials and accessors
         materials = self.remove_duplicates(self.materials)
         self.materials = []
@@ -608,15 +623,19 @@ class GLTF(object):
                         self.accessors.append(accessor)
                         # don't break here, there can be multiple accessors in a prim
 
-        # Keep only the most active animation TODO: have a flag for this
-        animations = self.remove_duplicates(self.animations)
-        self.animations = []
-        if len(animations) > 1:
-            most_active = animations[0]
-            for anim in animations[1:]:
-                if len(anim.channels) > len(most_active.channels):
-                    most_active = anim
-            self.animations.append(most_active)
+        # repair animations:
+        self.animations = self.remove_duplicates(self.animations)
+        for anim in self.animations:
+            anim.repair()
+
+        # Keep only most active animation
+        # animations = self.remove_duplicates(self.animations)
+        # if len(animations) >= 1:
+        #     most_active = animations[0]
+        #     for anim in animations[1:]:
+        #         if len(anim.channels) > len(most_active.channels):
+        #             most_active = anim
+        #     self.animations.append(most_active)
 
         # add accessors used by animations
         for animation in self.animations:
