@@ -6,7 +6,9 @@ import shutil
 from enum import Enum
 from uuid import uuid4
 from io import BytesIO
+import hashlib
 
+import requests
 from PIL import Image as PImage
 
 from .utils import BaseGLTFStructure, load_uri
@@ -34,6 +36,13 @@ class Image(object):
                     self.data = image['uri']
             except KeyError:
                 logger.warning('Image has no bufferView or uri')
+        elif isinstance(image, PImage.Image):
+            img_bytes = BytesIO()
+            if len(image.getbands()) == 4:
+                image.save(img_bytes, format='png')
+            else:
+                image.save(img_bytes, format='jpeg', quality=95)
+            self.data = img_bytes.getvalue()
         else:
             self.data = image
 
@@ -58,6 +67,23 @@ class Image(object):
         if type(data) is str:
             if data.startswith('data:'):
                 data = load_uri(data)
+            elif data.startswith('http'):
+                CACHE_DIR = '/tmp/py-gltf'
+                if not os.path.exists(CACHE_DIR):
+                    os.mkdir(CACHE_DIR)
+                cache_str = hashlib.md5(data.encode('utf-8')).hexdigest()
+                cache_path = os.path.join(CACHE_DIR, cache_str)
+                if os.path.exists(cache_path):
+                    with open(cache_path, 'rb') as f:
+                        data = f.read()
+                else:
+                    r = requests.get(data)
+                    if r.status_code >= 400:
+                        logger.warning('Unable to fetch image uri')
+                    else:
+                        data = r.content
+                        with open(cache_path, 'wb') as f:
+                            f.write(data)
             else:
                 data = data.replace('\\', '/')
                 if os.path.isabs(data):
@@ -151,7 +177,7 @@ class Image(object):
 
         return path
 
-    def render(self, buffer, gltf, embed=False, write_to=None):
+    def render(self, buffer, gltf, embed=False, write_to=None, splitting=False):
         image = {}
         if self.name:
             image['name'] = self.name
@@ -167,6 +193,8 @@ class Image(object):
             folder = os.path.dirname(write_to)
             name = os.path.basename(write_to)
             image['uri'] = self.write_to(folder, name + str(gltf.images.index(self)))
+            if splitting:
+                image['uri'] = os.path.basename(image['uri'])
             return image
 
         if self.is_uri:
@@ -323,17 +351,22 @@ class Material(BaseGLTFStructure):
     base_color_factor = None
     color_texture = None
     color_uv = None
+    color_extensions = None
     rough_metal_texture = None
     rough_uv = None
+    rough_metal_extensions = None
     normal_texture = None
     normal_uv = None
     normal_scale = None
+    normal_extensions = None
     occlusion_texture = None
     occlusion_uv = None
     occlusion_strength = None
+    occlusion_extensions = None
     emissive_texture = None
     emissive_factor = None
     emissive_uv = None
+    emissive_extensions = None
     metallic_factor = None
     roughness_factor = None
 
@@ -375,38 +408,48 @@ class Material(BaseGLTFStructure):
                 if base_color_texture:
                     self.color_texture = gltf.textures[base_color_texture['index']]
                     if 'texCoord' in base_color_texture:
-                        self.color_uv = gltf.accessors[base_color_texture['texCoord']]
+                        self.color_uv = base_color_texture['texCoord']
+                    if 'extensions' in base_color_texture:
+                        self.color_extensions = base_color_texture['extensions']
 
                 metallic_roughness_tex = pbr.get('metallicRoughnessTexture')
                 if metallic_roughness_tex:
                     self.rough_metal_texture = gltf.textures[metallic_roughness_tex['index']]
                     if 'texCoord' in metallic_roughness_tex:
-                        self.rough_uv = gltf.accessors[metallic_roughness_tex['texCoord']]
+                        self.rough_uv = metallic_roughness_tex['texCoord']
+                    if 'extensions' in metallic_roughness_tex:
+                        self.rough_metal_extensions = metallic_roughness_tex['extensions']
 
             normal = material.get('normalTexture')
             if normal:
                 if 'index' in normal:
                     self.normal_texture = gltf.textures[normal['index']]
                 if 'texCoord' in normal:
-                    self.normal_uv = gltf.accessors[normal['texCoord']]
+                    self.normal_uv = normal['texCoord']
                 if 'scale' in normal:
                     self.normal_scale = normal['scale']
+                if 'extensions' in normal:
+                    self.normal_extensions = normal['extensions']
 
             occlusion = material.get('occlusionTexture')
             if occlusion:
                 if 'index' in occlusion:
                     self.occlusion_texture = gltf.textures[occlusion['index']]
                 if 'texCoord' in occlusion:
-                    self.occlusion_uv = gltf.accessors[occlusion['texCoord']]
+                    self.occlusion_uv = occlusion['texCoord']
                 if 'strength' in occlusion:
                     self.occlusion_strength = occlusion['strength']
+                if 'extensions' in occlusion:
+                    self.occlusion_extensions = occlusion['extensions']
 
             emissive = material.get('emissiveTexture')
             if emissive:
                 if 'index' in emissive:
                     self.emissive_texture = gltf.textures[emissive['index']]
                 if 'texCoord' in emissive:
-                    self.emissive_uv = gltf.accessors[emissive['texCoord']]
+                    self.emissive_uv = emissive['texCoord']
+                if 'extensions' in emissive:
+                    self.emissive_extensions = emissive['extensions']
 
             extensions = material.get('extensions')
             if extensions:
@@ -469,44 +512,54 @@ class Material(BaseGLTFStructure):
             color_texture = {
                 'index': gltf.index('textures', self.color_texture)
             }
-            if self.color_uv:
-                color_texture['texCoord'] = gltf.index('accessors', self.color_uv)
+            if self.color_uv is not None:
+                color_texture['texCoord'] = self.color_uv
+            if self.color_extensions:
+                color_texture['extensions'] = self.color_extensions
             material['pbrMetallicRoughness']['baseColorTexture'] = color_texture
 
         if self.rough_metal_texture:
             rough_metal_texture = {
                 'index': gltf.index('textures', self.rough_metal_texture)
             }
-            if self.rough_uv:
-                rough_metal_texture['texCoord'] = gltf.index('accessors', self.rough_uv)
+            if self.rough_uv is not None:
+                rough_metal_texture['texCoord'] = self.rough_uv
+            if self.rough_metal_extensions:
+                rough_metal_texture['extensions'] = self.rough_metal_extensions
             material['pbrMetallicRoughness']['metallicRoughnessTexture'] = rough_metal_texture
 
         if self.normal_texture:
             normal_texture = {
                 'index': gltf.index('textures', self.normal_texture)
             }
-            if self.normal_uv:
-                normal_texture['texCoord'] = gltf.index('accessors', self.normal_uv)
+            if self.normal_uv is not None:
+                normal_texture['texCoord'] = self.normal_uv
             if self.normal_scale is not None:
                 normal_texture['scale'] = self.normal_scale
+            if self.normal_extensions:
+                normal_texture['extensions'] = self.normal_extensions
             material['normalTexture'] = normal_texture
 
         if self.occlusion_texture:
             occlusion_texture = {
                 'index': gltf.index('textures', self.occlusion_texture)
             }
-            if self.occlusion_uv:
-                occlusion_texture['texCoord'] = gltf.index('accessors', self.occlusion_uv)
+            if self.occlusion_uv is not None:
+                occlusion_texture['texCoord'] = self.occlusion_uv
             if self.occlusion_strength is not None:
                 occlusion_texture['strength'] = self.occlusion_strength
+            if self.occlusion_extensions:
+                occlusion_texture['extensions'] = self.occlusion_extensions
             material['occlusionTexture'] = occlusion_texture
 
         if self.emissive_texture:
             emissive_texture = {
                 'index': gltf.index('textures', self.emissive_texture)
             }
-            if self.emissive_uv:
-                emissive_texture['texCoord'] = gltf.index('accessors', self.emissive_uv)
+            if self.emissive_uv is not None:
+                emissive_texture['texCoord'] = self.emissive_uv
+            if self.emissive_extensions:
+                emissive_texture['extensions'] = self.emissive_extensions
             material['emissiveTexture'] = emissive_texture
 
         if self.name:
