@@ -8,7 +8,7 @@ from pyquaternion import Quaternion
 
 from .utils import BaseGLTFStructure
 from .materials import Material, Sampler
-from .buffers import Vec3Array, Vec2Array, Vec4Array, ScalarArray
+from .buffers import Vec3Array, Vec4Array, ScalarArray
 
 logger = logging.getLogger(__name__)
 
@@ -16,11 +16,12 @@ logger = logging.getLogger(__name__)
 class Primitive(BaseGLTFStructure):
     _positions = None
     _normals = None
-    _texcoords = None
     _indices = None
     _tangents = None
     mode = None
     material = None
+    texcoords = None
+    colors = None
     joints = None
     weights = None
 
@@ -35,7 +36,7 @@ class Primitive(BaseGLTFStructure):
 
     def __init__(self, prim=None, gltf=None, name=None, joints=None, weights=None,
                  positions=None, normals=None, indices=None, texcoords=None, tangents=None,
-                 material=None, image=None, texture=None, mode=None):
+                 material=None, image=None, texture=None, mode=None, colors=None):
         self.name = name
         if prim:
             if 'name' in prim:
@@ -54,8 +55,24 @@ class Primitive(BaseGLTFStructure):
                     self.normals = gltf.accessors[attributes.get('NORMAL')]
                 if 'TANGENT' in attributes:
                     self.tangents = gltf.accessors[attributes.get('TANGENT')]
+                self.texcoords = []
                 if 'TEXCOORD_0' in attributes:
-                    self.texcoords = gltf.accessors[attributes.get('TEXCOORD_0')]
+                    i = 0
+                    while True:
+                        texcoords = attributes.get('TEXCOORD_' + str(i))
+                        if texcoords is None:
+                            break
+                        self.texcoords.append(gltf.accessors[texcoords])
+                        i += 1
+                self.colors = []
+                if 'COLOR_0' in attributes:
+                    i = 0
+                    while True:
+                        colors = attributes.get('COLOR_' + str(i))
+                        if colors is None:
+                            break
+                        self.colors.append(gltf.accessors[colors])
+                        i += 1
                 self.joints = []
                 if 'JOINTS_0' in attributes:
                     i = 0
@@ -79,8 +96,9 @@ class Primitive(BaseGLTFStructure):
             self.normals = normals
             self.indices = indices
             self.tangents = tangents
-            self.texcoords = texcoords
             self.material = material
+            self.texcoords = texcoords if texcoords is not None else []
+            self.colors = colors if colors is not None else []
             self.joints = joints if joints is not None else []
             self.weights = weights if weights is not None else []
             self.mode = mode and self.Mode[mode]
@@ -95,9 +113,12 @@ class Primitive(BaseGLTFStructure):
     def __contains__(self, item):
         return (self._positions == item or
                 self._normals == item or
-                self._texcoords == item or
                 self._indices == item or
-                self._tangents == item)
+                self._tangents == item or
+                item in self.texcoords or
+                item in self.colors or
+                item in self.joints or
+                item in self.weights)
 
     def sort_joints(self):
         if len(self.joints) > 1:
@@ -146,6 +167,11 @@ class Primitive(BaseGLTFStructure):
         if min_alpha == 255 or max_alpha < 255:
             return
 
+        if self.material.color_uv is not None:
+            texcoords = self.texcoords[self.material.color_uv]
+        else:
+            texcoords = self.texcoords[0]
+
         if self.indices:
             indices_iter = (self.indices.data[pos:pos + 3]
                             for pos in
@@ -153,11 +179,11 @@ class Primitive(BaseGLTFStructure):
         else:
             indices_iter = ((i, i + 1, i + 2)
                             for i in
-                            range(0, self.texcoords.count, 3))
+                            range(0, texcoords.count, 3))
         t_indices = []
         o_indices = []
         for indices in indices_iter:
-            for point in [self.texcoords.data[i] for i in indices]:
+            for point in [texcoords.data[i] for i in indices]:
                 point = sampler.wrap_point(point)
                 x = round((alpha.size[0] - 1) * point[0])
                 y = round((alpha.size[1] - 1) * point[1])
@@ -182,6 +208,7 @@ class Primitive(BaseGLTFStructure):
 
         return Primitive(positions=self.positions, normals=self.normals,
                          texcoords=self.texcoords, tangents=self.tangents,
+                         joints=self.joints, weights=self.weights, colors=self.colors,
                          indices=t_indices, material=t_material)
 
     def render(self, gltf):
@@ -198,8 +225,10 @@ class Primitive(BaseGLTFStructure):
             primitive['attributes']['NORMAL'] = gltf.index('accessors', self.normals)
         if self.tangents and self.tangents.count:
             primitive['attributes']['TANGENT'] = gltf.index('accessors', self.tangents)
-        if self.texcoords and self.texcoords.count:
-            primitive['attributes']['TEXCOORD_0'] = gltf.index('accessors', self.texcoords)
+        for i, texcoords in enumerate(self.texcoords):
+            primitive['attributes']['TEXCOORD_' + str(i)] = gltf.index('accessors', texcoords)
+        for i, colors in enumerate(self.colors):
+            primitive['attributes']['COLOR_' + str(i)] = gltf.index('accessors', colors)
         for i, joints in enumerate(self.joints):
             primitive['attributes']['JOINTS_' + str(i)] = gltf.index('accessors', joints)
         for i, weights in enumerate(self.weights):
@@ -239,17 +268,6 @@ class Primitive(BaseGLTFStructure):
             return
         self._tangents = tangents if isinstance(tangents, Vec4Array) else Vec4Array(tangents)
 
-    @property
-    def texcoords(self):
-        return self._texcoords
-
-    @texcoords.setter
-    def texcoords(self, texcoords):
-        if texcoords is None:
-            self._texcoords = None
-            return
-        self._texcoords = texcoords if isinstance(texcoords, Vec2Array) else Vec2Array(texcoords)
-
     @property
     def indices(self):
         return self._indices
